<?php
/**
 * Plugin Name: WP Stripe Confirmation Page
 * Description: Enhanced confirmation page for your Stripe transactions
 * Version: 1.0.1
 * Author: HeadwayApps
 * Author URI: https://headwayapps.com/
 */

namespace HeadwayApps;

class StripeClient
{
	const API_BASE_URL = 'https://api.stripe.com/v1';

	private $secretKey;

	public function __construct()
	{
		$options = get_option( 'headwayapps_stripe_confirmation_options' );
		if (is_array($options) && isset($options['secret_key'])) {
			$this->secretKey = openssl_decrypt( $options['secret_key'], 'aes256', SECURE_AUTH_SALT );
		}
	}

	public function get_session($session_id)
	{
		if ($this->secretKey) {
			return json_decode( $this->_get('/checkout/sessions/' . $session_id)['body'], true );
		}
	}

	public function get_customer($customer_id)
	{
		if ($this->secretKey) {
			return json_decode( $this->_post('/customers/' . $customer_id)['body'], true );
		}
	}

	public function get_payment_intent($payment_intent_id)
	{
		if ($this->secretKey) {
			return json_decode( $this->_get('/payment_intents/' . $payment_intent_id)['body'], true );
		}
	}

	public function get_subscription($subscription_id)
	{
		if ($this->secretKey) {
			return json_decode( $this->_get('/subscriptions/' . $subscription_id)['body'], true );
		}
	}

	public function get_invoice($invoice_id)
	{
		if ($this->secretKey) {
			return json_decode( $this->_get('/invoices/' . $invoice_id)['body'], true );
		}
	}

	private function _post($path)
	{
		return wp_remote_post( self::API_BASE_URL . $path, $this->_get_auth_headers() );
	}

	private function _get($path)
	{
		return wp_remote_get( self::API_BASE_URL . $path, $this->_get_auth_headers() );
	}

	private function _get_auth_headers()
	{
		return [
			'headers' => [
				'Authorization' => 'Basic '. base64_encode($this->secretKey .':'),
			]
		];
	}
}

class StripeConfirmation
{
	const SLUG = 'headwayapps_stripe_confirmation';
	const SETTINGS_SLUG = self::SLUG . '_settings';
	const OPTIONS_SLUG = self::SLUG . '_options';

	private $stripeClient;
	private $hasShortcodeOnPage;
	private $variablesWhitelist = [
		'customer' => [
			'CUSTOMER_NAME' => 'name',
			'CUSTOMER_EMAIL' => 'email',
		],
		'session' => [
			'TOTAL_AMOUNT' => 'amount_total'
		],
		'invoice' => [
			'TRANSACTION_ID' => 'id',
			'DOWNLOAD_RECEIPT_AND_INVOICE_URL' => 'hosted_invoice_url',
			'DATE' => 'created'
		],
	];

	public function __construct()
	{
		$this->stripeClient = new StripeClient();

		register_activation_hook( __FILE__, [ &$this, 'set_default_options' ] );

		add_action( 'admin_menu', [ &$this, 'register_menu' ], 20 );
		add_action( 'admin_init', [ &$this, 'register_settings' ] );
		add_shortcode( 'stripe-confirmation', [ &$this, 'render_stripe_confirmation_shortcode' ] );
		add_action( 'wp_head', [ &$this, 'add_header_html' ] );
		add_action( 'wp_footer', [ &$this, 'add_footer_html' ] );
		add_action( 'init', [ &$this, 'register_params' ] );

		add_filter( 'content_html', 'wptexturize' );
		add_filter( 'content_html', 'convert_smilies' );
		add_filter( 'content_html', 'convert_chars' );
		add_filter( 'content_html', 'wpautop' );
		add_filter( 'content_html', 'shortcode_unautop' );
		add_filter( 'content_html', 'prepend_attachment' );
		add_filter( 'content_html', 'do_shortcode' );
	}

	public function get_stripe_client()
	{
		return $this->stripeClient;
	}

	public function register_menu()
	{
		add_options_page(
			'HeadwayApps Stripe Confirmation Page',
			'Stripe Confirmation Page Settings',
			'manage_options',
			self::SETTINGS_SLUG,
			[ &$this, 'render_plugin_settings_page' ]
		);
	}

	function register_settings()
	{
		register_setting(
			self::OPTIONS_SLUG,
			self::OPTIONS_SLUG,
			[ &$this, 'encrypt_secret_key_on_save' ]
		);

		add_settings_section(
			'api_settings',
			'API Settings',
			[ &$this, 'render_plugin_settings_api_section' ],
			self::OPTIONS_SLUG
		);

		add_settings_section(
			'template_settings',
			'Template Settings',
			[ &$this, 'render_plugin_settings_template_section' ],
			self::OPTIONS_SLUG
		);

		add_settings_field(
			'secret_key',
			'Secret Key',
			[ &$this, 'render_plugin_setting_secret_key' ],
			self::OPTIONS_SLUG,
			'api_settings'
		);

		add_settings_field(
			'session_id_param',
			'Session ID Param Name',
			[ &$this, 'render_plugin_setting_session_id_param' ],
			self::OPTIONS_SLUG,
			'api_settings'
		);
		add_settings_field(
			'header_html',
			'Header HTML',
			[ &$this, 'render_plugin_setting_header_html' ],
			self::OPTIONS_SLUG,
			'template_settings'
		);
		add_settings_field(
			'content_html',
			'Content HTML',
			[ &$this, 'render_plugin_setting_content_html' ],
			self::OPTIONS_SLUG,
			'template_settings'
		);
		add_settings_field(
			'error_content_html',
			'Integration Error Content HTML',
			[ &$this, 'render_plugin_setting_error_content_html' ],
			self::OPTIONS_SLUG,
			'template_settings'
		);
		add_settings_field(
			'footer_html',
			'Footer HTML',
			[ &$this, 'render_plugin_setting_footer_html' ],
			self::OPTIONS_SLUG,
			'template_settings'
		);
	}

	public function set_default_options()
	{
		$options = get_option( 'headwayapps_stripe_confirmation_options' );
		if (! $options) {
			update_option( 'headwayapps_stripe_confirmation_options', [
				'content_html' => '<p>Hello %CUSTOMER_NAME%,</p>
<p>&nbsp;</p>
<p>Here\'s the receipt for your payment:</p>
<p>
	Transaction ID: %TRANSACTION_ID%<br/>
	Customer Name: %CUSTOMER_NAME%<br/>
	Customer Email: %CUSTOMER_EMAIL%<br/>
	Amount: %TOTAL_AMOUNT%
</p>
<p><a href="%DOWNLOAD_RECEIPT_AND_INVOICE_URL%" target="_blank" rel="noopener">Download receipt and invoice</a></p>

<p>&nbsp;</p>

<p>Thank you for your business!</p>

<p>Cheers,<br/>
Site Name</p>'
			]);
		}
	}

	public function render_plugin_settings_page()
	{
		include __DIR__ . '/templates/admin/settings-page.php';
	}

	public function render_plugin_settings_api_section()
	{
		include __DIR__ . '/templates/admin/settings-api-section.php';
	}

	public function render_plugin_settings_template_section()
	{
		include __DIR__ . '/templates/admin/settings-template-section.php';
	}

	public function render_plugin_setting_secret_key()
	{
		include __DIR__ . '/templates/admin/settings-secret-key.php';
	}

	public function render_plugin_setting_session_id_param()
	{
		include __DIR__ . '/templates/admin/settings-session-id-param.php';
	}

	public function render_plugin_setting_header_html()
	{
		include __DIR__ . '/templates/admin/settings-header-html.php';
	}

	public function render_plugin_setting_content_html()
	{
		include __DIR__ . '/templates/admin/settings-content-html.php';
	}

	public function render_plugin_setting_error_content_html()
	{
		include __DIR__ . '/templates/admin/settings-error-content-html.php';
	}

	public function render_plugin_setting_footer_html()
	{
		include __DIR__ . '/templates/admin/settings-footer-html.php';
	}

	public function render_stripe_confirmation_shortcode()
	{
		$options = get_option( 'headwayapps_stripe_confirmation_options' );
		$content = is_array($options) && isset($options['content_html']) ? $options['content_html'] : '';
		$errorContent = is_array($options) && isset($options['error_content_html']) ? $options['error_content_html'] : '';

		return $this->render_header_footer_html($content, $errorContent);
	}

	public function encrypt_secret_key_on_save($input)
	{
		$options = get_option( 'headwayapps_stripe_confirmation_options' );
		if (is_array($options)
		    && (empty($options['secret_key'])
		              || (! empty($options['secret_key']) && $options['secret_key'] !== $input['secret_key']))) {
			$input['secret_key'] = openssl_encrypt( $input['secret_key'], 'aes256', SECURE_AUTH_SALT );
		}
		return $input;
	}

	public function add_header_html()
	{
		if ( $this->has_shortcode_on_current_page() ) {
			$options = get_option( 'headwayapps_stripe_confirmation_options' );
			echo is_array($options) && ! empty($options['header_html'])
				? $this->render_header_footer_html( $options['header_html'] )
				: '';
		}
	}

	public function add_footer_html()
	{
		if ( $this->has_shortcode_on_current_page() ) {
			$options = get_option( 'headwayapps_stripe_confirmation_options' );
			echo is_array($options) && ! empty($options['footer_html'])
				? $this->render_header_footer_html( $options['footer_html'] )
				: '';
		}
	}

	private function render_header_footer_html($content, $errorContent = null)
	{
		if ( $session_id = get_query_var( $this->get_session_id_param_name() ) ) {
			$session = $this->get_stripe_client()->get_session($session_id);
			if (empty($session['error'])) {
				$customer = $this->get_stripe_client()->get_customer($session['customer']);
				if ($session['mode'] === 'subscription') {
					$subscription = $this->get_stripe_client()->get_subscription($session['subscription']);
					$invoiceId = $subscription['latest_invoice'];
				} else {
					$paymentIntent = $this->get_stripe_client()->get_payment_intent($session['payment_intent']);
					$invoiceId = $paymentIntent['invoice'];
				}
				$invoice = $this->get_stripe_client()->get_invoice($invoiceId);
				if (empty($customer['error'])) {
					foreach ($this->variablesWhitelist['customer'] as $customerKey => $customerIndex) {
						$content = str_replace('%' . $customerKey . '%', $customer[$customerIndex], $content);
					}
				}
				if (empty($invoice['error'])) {
					foreach ($this->variablesWhitelist['invoice'] as $iKey => $iIndex) {
						$iValue = ($iIndex === 'created')
							? date('Y-m-d H:i:s', $invoice[$iIndex])
							: $invoice[$iIndex];
						$content = str_replace('%' . $iKey . '%', $iValue, $content);
					}
				}
				foreach ($this->variablesWhitelist['session'] as $sessionKey => $sessionIndex) {
					$sessionValue = $session[$sessionIndex] ?: 0;
					if ($sessionKey === 'TOTAL_AMOUNT') {
						$sessionValue = $sessionValue / 100;
					}
					$content = str_replace('%' . $sessionKey . '%', $sessionValue, $content);
				}
				return apply_filters( 'content_html', $content );
			}
			return apply_filters( 'content_html', $errorContent );
		}

		return '';
	}

	private function has_shortcode_on_current_page()
	{
		if ($this->hasShortcodeOnPage === null) {
			global $post;
			$this->hasShortcodeOnPage = is_a( $post, 'WP_Post' ) && has_shortcode( $post->post_content, 'stripe-confirmation');
		}

		return $this->hasShortcodeOnPage;
	}

	public function register_params()
	{
		global $wp;
		$wp->add_query_var($this->get_session_id_param_name());
	}

	private function get_session_id_param_name()
	{
		$options = get_option( 'headwayapps_stripe_confirmation_options' );
		$param = 'session_id';
		if (is_array($options) && ! empty($options['session_id_param'])) {
			$param = $options['session_id_param'];
		}
		return $param;
	}
}

$GLOBALS['headwayapps_stripe_confirmation'] = new StripeConfirmation();
